@@ -1,7 +1,6 @@
 import math
 from db_hj3415 import mongo2
 from typing import Tuple, List
-from util_hj3415 import utils
 from collections import OrderedDict
 
 import logging
@@ -10,7 +9,89 @@ formatter = logging.Formatter('%(levelname)s: [%(name)s] %(message)s')
 ch = logging.StreamHandler()
 ch.setFormatter(formatter)
 logger.addHandler(ch)
-logger.setLevel(logging.WARNING)
+logger.setLevel(logging.INFO)
+
+
+class C103(mongo2.C103):
+    """
+    mongo2의 C103에서 title이 없는 경우에 nan을 반환하도록 처리하기 위해 상속함.
+    """
+    def latest_value(self, title: str, pop_count=2, nan_to_zero=False) -> Tuple[str, float]:
+        """ 몽고 데이터베이스에서 title 항목이 없는 경우를 처리하기위해 오버라이드함.
+        """
+        try:
+            date, value = super(C103, self).latest_value(title, pop_count)
+        except:
+            date, value = '', float('nan')
+        logger.debug(f'{title} : {date}\t{value}')
+        if nan_to_zero and math.isnan(value):
+            return date, 0
+        else:
+            return date, value
+
+    def sum_recent_4q(self, title: str, nan_to_zero=False) -> Tuple[str, float]:
+        """ 몽고 데이터베이스에서 title 항목이 없는 경우를 처리하기위해 오버라이드함.
+                """
+        try:
+            date, value = super(C103, self).sum_recent_4q(title)
+        except:
+            date, value = '', float('nan')
+        logger.debug(f'{title} : {date}\t{value}')
+        if nan_to_zero and math.isnan(value):
+            return date, 0
+        else:
+            return date, value
+
+    def find(self, title: str, allow_empty=False) -> dict:
+        try:
+            rdict = super(C103, self).find(title)
+        except Exception as e:
+            if allow_empty:
+                rdict = {}
+            else:
+                raise Exception(e)
+        return rdict
+
+
+class C104(mongo2.C104):
+    """
+    mongo2의 C104에서 title이 없는 경우에 nan을 반환하도록 처리하기 위해 상속함.
+    """
+    def latest_value(self, title: str, pop_count=2, nan_to_zero=False) -> Tuple[str, float]:
+        """ 몽고 데이터베이스에서 title 항목이 없는 경우를 처리하기위해 오버라이드함.
+        """
+        try:
+            date, value = super(C104, self).latest_value(title, pop_count)
+        except:
+            date, value = '', float('nan')
+        logger.debug(f'{title} : {date}\t{value}')
+        if nan_to_zero and math.isnan(value):
+            return date, 0
+        else:
+            return date, value
+
+    def sum_recent_4q(self, title: str, nan_to_zero=False) -> Tuple[str, float]:
+        """ 몽고 데이터베이스에서 title 항목이 없는 경우를 처리하기위해 오버라이드함.
+                """
+        try:
+            date, value = super(C104, self).sum_recent_4q(title)
+        except:
+            date, value = '', float('nan')
+        logger.debug(f'{title} : {date}\t{value}')
+        if nan_to_zero and math.isnan(value):
+            return date, 0
+        else:
+            return date, value
+
+    def find(self, title: str, allow_empty=False) -> dict:
+        try:
+            rdict = super(C104, self).find(title)
+        except Exception as e:
+            if allow_empty:
+                rdict = {}
+            else:
+                raise Exception(e)
+        return rdict
 
 
 class CorpsEval:
@@ -18,30 +99,26 @@ class CorpsEval:
 
     """
 
-    def __init__(self,client, code: str):
-        if utils.is_6digit(code):
-            self.client = client
-            self.code = code
-            self.c101 = mongo2.C101(self.client, code)
-            self.c103 = mongo2.C103(self.client, code, 'c103재무상태표y')
-            self.c104 = mongo2.C104(self.client, code, 'c104y')
-        else:
-            raise ValueError(f'Invalid code : {code}')
-        # C103이 억단위로 계산되기 때문에 시가총액도 억단위로 맞춘다.
-        try:
-            self.marketcap = int(self.c101.get_recent()['시가총액']) / 100000000
-        except KeyError:
-            self.marketcap = float('nan')
+    def __init__(self, client, code: str):
+        self.client = client
+        self.c101 = mongo2.C101(self.client, code)
+        self.c103 = C103(self.client, code, 'c103재무상태표y')
+        self.c104 = C104(self.client, code, 'c104y')
+        self._code = None
+        self.marketcap = None
+        self.code = code
+
+    @property
+    def code(self):
+        return self._code
+
+    @code.setter
+    def code(self, code):
+        self.c101.code = code
+        self.c103.code = code
+        self.c104.code = code
+        self._code = code
 
-    def chg_code(self, code: str):
-        if utils.is_6digit(code):
-            self.code = code
-            self.c101.code = code
-            self.c103.code = code
-            self.c104.code = code
-        else:
-            raise ValueError(f'Invalid code : {code}')
-        # C103이 억단위로 계산되기 때문에 시가총액도 억단위로 맞춘다.
         try:
             self.marketcap = int(self.c101.get_recent()['시가총액']) / 100000000
         except KeyError:
@@ -59,12 +136,13 @@ class CorpsEval:
         Returns:
             Tuple: (date, 총합 value) 형식의 튜플
         """
-        logger.info(f'sum_except_nan : {targets}')
+        logger.debug(f'sum_except_nan : {targets}')
         s_date = set()
         t_value = 0
         for date, value in targets:
-            s_date.add(date)
-            t_value += value
+            if date != '':
+                s_date.add(date)
+            t_value += 0 if math.isnan(value) else value
         try:
             return list(s_date)[0], round(t_value, 2)
         except IndexError:
@@ -78,20 +156,18 @@ class CorpsEval:
         금융기관의 경우는 지배당기순이익이 없기 때문에\n
         계산을 통해서 간접적으로 구한다.\n
         """
-        self.c103.page = 'c103재무상태표q'
-        profit_dict = self.c103.find(title='*(지배)당기순이익')
-        logger.info(f'*(지배)당기순이익 : {profit_dict}')
-        if len(profit_dict) == 0:
+        try:
+            self.c103.page = 'c103재무상태표q'
+            profit_dict = self.c103.find(title='*(지배)당기순이익')
+            logger.debug(f'*(지배)당기순이익 : {profit_dict}')
+            return self.c103.latest_value('*(지배)당기순이익')
+        except:
             # 금융관련은 재무상태표에 지배당기순이익이 없어서 손익계산서의 당기순이익에서 비지배당기순이익을 빼서 간접적으로 구한다.
             self.c103.page = 'c103손익계산서q'
             t1 = self.c103.sum_recent_4q('당기순이익')
-            logger.info(f'당기순이익(손익계산서) : {t1[0]}\t{t1[1]}')
             self.c103.page = 'c103재무상태표q'
             t2 = self.c103.latest_value('*(비지배)당기순이익')
-            logger.info(f'*(비지배)당기순이익 : {t2[0]}\t{t2[1]}')
             return self.sum_except_nan([t1, t2])
-        else:
-            return self.c103.latest_value('*(지배)당기순이익')
 
     def calc유동자산(self) -> Tuple[str, float]:
         """유효한 유동자산 계산
@@ -100,22 +176,18 @@ class CorpsEval:
         금융기관의 경우는 간접적으로 계산한다.\n
         Red와 Blue에서 사용한다.\n
         """
-        self.c103.page = 'c103재무상태표q'
-        asset_dict = self.c103.find(title='유동자산')
-        logger.info(f'유동자산 : {asset_dict}')
-        if len(asset_dict) == 0:
+        try:
+            self.c103.page = 'c103재무상태표q'
+            asset_dict = self.c103.find(title='유동자산')
+            logger.debug(f'유동자산 : {asset_dict}')
+            return self.c103.sum_recent_4q('유동자산')
+        except:
             # 금융관련업종...
             t1 = self.c103.latest_value('현금및예치금')
-            logger.info(f'현금및예치금 : {t1[0]}\t{t1[1]}')
             t2 = self.c103.latest_value('단기매매금융자산')
-            logger.info(f'단기매매금융자산 : {t2[0]}\t{t2[1]}')
             t3 = self.c103.latest_value('매도가능금융자산')
-            logger.info(f'매도가능금융자산 : {t3[0]}\t{t3[1]}')
             t4 = self.c103.latest_value('만기보유금융자산')
-            logger.info(f'만기보유금융자산 : {t4[0]}\t{t4[1]}')
             return self.sum_except_nan([t1, t2, t3, t4])
-        else:
-            return self.c103.sum_recent_4q('유동자산')
 
     def calc유동부채(self) -> Tuple[str, float]:
         """유효한 유동부채 계산
@@ -124,22 +196,18 @@ class CorpsEval:
         금융기관의 경우는 간접적으로 계산한다.\n
         Red와 Blue에서 사용한다.\n
         """
-        self.c103.page = 'c103재무상태표q'
-        debt_dict = self.c103.find(title='유동부채')
-        logger.info(f'유동부채 : {debt_dict}')
-        if len(debt_dict) == 0:
+        try:
+            self.c103.page = 'c103재무상태표q'
+            debt_dict = self.c103.find(title='유동부채')
+            logger.debug(f'유동부채 : {debt_dict}')
+            return self.c103.sum_recent_4q('유동부채')
+        except:
             # 금융관련업종...
             t1 = self.c103.latest_value('당기손익인식(지정)금융부채')
-            logger.info(f'당기손익인식(지정)금융부채 : {t1[0]}\t{t1[1]}')
             t2 = self.c103.latest_value('당기손익-공정가치측정금융부채')
-            logger.info(f'당기손익-공정가치측정금융부채 : {t2[0]}\t{t2[1]}')
             t3 = self.c103.latest_value('매도파생결합증권')
-            logger.info(f'매도파생결합증권 : {t3[0]}\t{t3[1]}')
             t4 = self.c103.latest_value('단기매매금융부채')
-            logger.info(f'단기매매금융부채 : {t4[0]}\t{t4[1]}')
             return self.sum_except_nan([t1, t2, t3, t4])
-        else:
-            return self.c103.sum_recent_4q('유동부채')
 
     def calc비유동부채(self) -> Tuple[str, float]:
         """유효한 비유동부채 계산
@@ -148,33 +216,29 @@ class CorpsEval:
         금융기관의 경우는 간접적으로 계산한다.\n
         Red와 Blue에서 사용한다.\n
         """
-        self.c103.page = 'c103재무상태표q'
-        debt_dict = self.c103.find(title='비유동부채')
-        logger.info(f'유동부채 : {debt_dict}')
-        if len(debt_dict) == 0:
+        try:
+            self.c103.page = 'c103재무상태표q'
+            debt_dict = self.c103.find(title='비유동부채')
+            logger.debug(f'비유동부채 : {debt_dict}')
+            return self.c103.sum_recent_4q('비유동부채')
+        except:
             # 금융관련업종...
             # 보험관련업종은 예수부채가 없는대신 보험계약부채가 있다...
             t1 = self.c103.latest_value('예수부채')
-            logger.info(f'예수부채 : {t1[0]}\t{t1[1]}')
             t2 = self.c103.latest_value('보험계약부채(책임준비금)')
-            logger.info(f'보험계약부채(책임준비금) : {t2[0]}\t{t2[1]}')
             t3 = self.c103.latest_value('차입부채')
-            logger.info(f'차입부채 : {t3[0]}\t{t3[1]}')
             t4 = self.c103.latest_value('기타부채')
-            logger.info(f'기타부채 : {t4[0]}\t{t4[1]}')
             return self.sum_except_nan([t1, t2, t3, t4])
-        else:
-            return self.c103.sum_recent_4q('비유동부채')
 
-    def calc유동비율(self, nan_to_zero: bool = True) -> Tuple[str, float]:
+    def calc유동비율(self, pop_count) -> Tuple[str, float]:
         """유동비율계산 - Blue에서 사용
 
         c104q에서 최근유동비율 찾아보고 유효하지 않거나 \n
         100이하인 경우에는수동으로 계산해서 다시 한번 평가해 본다.\n
         """
         self.c104.page = 'c104q'
-        d1, 유동비율 = self.c104.latest_value('유동비율')
-        logger.info(f'{self.code} 유동비율 : {유동비율}')
+        d1, 유동비율 = self.c104.latest_value('유동비율', pop_count=pop_count)
+        logger.debug(f'{self.code} 유동비율 : {유동비율}')
 
         if math.isnan(유동비율) or 유동비율 < 100:
             logger.warning('유동비율 is under 100 or nan..so we will recalculate..')
@@ -184,16 +248,21 @@ class CorpsEval:
             self.c103.page = 'c103현금흐름표q'
             d3, 추정영업현금흐름 = self.c103.sum_recent_4q('영업활동으로인한현금흐름')
 
+            logger.debug(f'{self.code} 계산전 유동비율 : {유동비율}')
+            계산된유동비율 = 0
             try:
-                logger.info(f'{self.code} 계산전 유동비율 : {유동비율}')
                 계산된유동비율 = round(((유동자산 + 추정영업현금흐름) / 유동부채) * 100, 2)
-                logger.info(f'{self.code} 계산된 유동비율 : {계산된유동비율}')
-                return list({d1, d2, d3})[0], 계산된유동비율
             except ZeroDivisionError:
-                if nan_to_zero:
-                    return list({d1, d2, d3})[0], 0
+                logger.debug(f'유동자산: {유동자산} + 추정영업현금흐름: {추정영업현금흐름} / 유동부채: {유동부채}')
+                if 유동자산 + 추정영업현금흐름 == float('nan') or 유동자산 + 추정영업현금흐름 == 0:
+                    계산된유동비율 = float('nan')
+                elif 유동자산 + 추정영업현금흐름 < 0:
+                    계산된유동비율 = float('-inf')
                 else:
-                    return list({d1, d2, d3})[0], float('nan')
+                    계산된유동비율 = float('inf')
+            finally:
+                logger.debug(f'{self.code} 계산된 유동비율 : {계산된유동비율}')
+                return list({d1, d2, d3})[0], 계산된유동비율
         else:
             return d1, 유동비율
 
@@ -213,7 +282,10 @@ class CorpsEval:
         self.c103.page = 'c103현금흐름표y'
         영업활동으로인한현금흐름 = self.c103.find(title='영업활동으로인한현금흐름')
         self.c103.page = 'c103재무상태표y'
-        capex = self.c103.find(title='*CAPEX')
+        try:
+            capex = self.c103.find(title='*CAPEX')
+        except:
+            capex = {}
 
         logger.debug(f'영업활동으로인한현금흐름 {영업활동으로인한현금흐름}')
         logger.debug(f'CAPEX {capex}')
@@ -271,7 +343,7 @@ class CorpsEval:
         # 딕셔너리 정렬 - https://kkamikoon.tistory.com/138
         # reverse = False 이면 오래된것부터 최근순으로 정렬한다.
         od = OrderedDict(sorted(data.items(), reverse=False))
-        logger.info(f'{od}')
+        logger.debug(f'{od}')
         try:
             last_one = od.popitem(last=True)
         except KeyError:
@@ -281,7 +353,7 @@ class CorpsEval:
             else:
                 return '', float('nan')
 
-        logger.info(f'last_one : {last_one}')
+        logger.debug(f'last_one : {last_one}')
         if isinstance(last_one[1], str):
             # last_one : ('Unnamed: 1', '데이터가 없습니다.') 인 경우
             if nan_to_zero:
